#!/bin/bash
#
# Convenience script for flashing Ubuntu phones
# and making common configurations.
#
# recovery images:  (see 'urls' file)

WIFI=no
MEDIA=no
EXTRA=
PASSWD=0000
DEVMODE="--developer-mode --password=$PASSWD"
ADB=yes
GU=no
ASK=yes
WIZARD=yes
EDGEINTRO=yes
APTUPDATE=no
BEEP=yes
DEVTAR=no
CHANNEL_PATTERN="rc-proposed"

for arg in $* ; do
  case "$arg" in
    --help)        # show CLI options and exit
      echo "Flashing tool."
      echo "Example use:"
      echo "  ANDROID_SERIAL=foo ./flash.sh --network --channel rc arale 217"
      echo "Command line options include:"
      cat "$0" | awk '/^for arg/,/^  esac/' | egrep '^ +--' | sed 's/)//;'
      shift
      exit 0
      ;;
    --network)     # configure wifi automatically (via phablet-network)
      WIFI=yes
      ADB=yes
      shift
      ;;
    --media)       # copy music/videos/photos to device
      MEDIA=yes
      ADB=yes
      shift
      ;;
    --nodm)        # don't enable developer mode
      DEVMODE=""
      ADB=no
      shift
      ;;
    --gu)          # override GU / graphical unit size
      GU=yes
      ADB=yes
      shift
      ;;
    --yes)         # don't ask for confirmation
      ASK=no
      shift
      ;;
    --nowizard)    # disable welcome wizard
      WIZARD=no
      shift
      ;;
    --nointro)     # disable edges intro
      EDGEINTRO=no
      shift
      ;;
    --apt-update)  # apt-get update after flashing
      APTUPDATE=yes
      shift
      ;;
    --channel)     # egrep for a matching image feed/channel and use it
      shift
      CHANNEL_PATTERN=$1
      shift
      ;;
    --nobeep)      # don't beep on completion
      BEEP=no
      shift
      ;;
    --devtar)      # use a special device tarball
      shift
      DEVTAR="$1"
      shift
      ;;
    --device)      # specify the device type / model (positional)
      shift
      MODEL=$1
      shift
      ;;
    --rev)         # specify the image number / revision to flash (positional)
      shift
      REVISION=$1
      shift
      ;;
    *)
      #echo "Arg: $arg"
      ;;
  esac
done
# Positional arguments
if [ x"$MODEL" = x ]; then MODEL=$1 ; shift ; fi
if [ x"$REVISION" = x ]; then REVISION=$1 ; shift ; fi

REVARGS=
SERVER=
CHANNEL=ubuntu-touch/stable/ubuntu
FASTBOOT=no
RECOVERY=

# Figure out the channel and other options based on the phone type
if [ x"$MODEL" = x ]; then
  echo "No device model specified.  Stopping."
  exit 1
fi
echo "Model: $MODEL"
if [ x"$REVISION" != x ]; then
  echo "Revision: $REVISION"
  REVARGS="--revision=$REVISION"
fi

# TODO: Also support custom tarballs.
# Use a special device tarball?
if [ x"$DEVTAR" = x"no" ]; then
  DEVTAR=""
else
  DEVTAR="--device-tarball $DEVTAR --run-script ./disable-gpg"
fi

# grab the device's channel from channels.lst or the server
if [ x"$MODEL" != x ]; then
  # let the user specify a channel like 'rc' or 'rc-proposed' or 'dev'
  #echo 'detecting channel by pattern'
  CHANNEL=$(
    grep "^$MODEL " channels.lst \
      | awk '{ print $2 }' \
      | grep "$CHANNEL_PATTERN" \
      | head -1
    )

  # in case the user specified a channel not in the list
  if [ x"$CHANNEL" = x ]; then
    echo 'asking server for matching channel'
    #CHANNEL="$CHANNEL_PATTERN"
    CHANNEL=$(ubuntu-device-flash query --device "$MODEL" --list-channels \
      | awk '{print length"\t"$0}' \
      | sort -n \
      | cut -f2- \
      | grep "$CHANNEL_PATTERN" \
      | head -1
      )
  fi

  # if all else fails, pick the shortest channel name
  if [ x"$CHANNEL" = x ]; then
    echo 'looking for default channel'
    # show all channels, to help people find what they need
    echo "All channels:"
    ubuntu-device-flash query --device "$MODEL" --list-channels | sort
    # pick the shortest one
    CHANNEL=$(ubuntu-device-flash query --device "$MODEL" --list-channels \
      | awk '{print length"\t"$0}' \
      | sort -n \
      | cut -f2- \
      | head -1
      )
  fi
fi

# detect the recovery image
if [ -e "$MODEL/recovery.img" ]; then
  RECOVERY="--recovery-image $MODEL/recovery.img"
fi

# device-specific options
if [ x"$MODEL" = x"mako" ]; then
  FASTBOOT=no
fi

if [ x"$MODEL" = x"krillin" ]; then
  FASTBOOT=yes
fi

if [ x"$MODEL" = xarale ]; then
  if [ x"$GU" = x"yes" ]; then
    GU=21
  fi

  # Public server
  MODEL=arale
  FASTBOOT=yes
fi

# Summarize what we're going to do
echo "====="
echo "Server: $SERVER"
echo "Channel: $CHANNEL"
echo "Fastboot: $FASTBOOT"
echo "Recovery: $RECOVERY"
echo "Wifi: $WIFI"
echo "Media: $MEDIA"
echo "Wizard/Intro: $WIZARD / $EDGEINTRO"
echo "apt-get update: $APTUPDATE"
echo "====="
echo ubuntu-device-flash $SERVER $REVARGS query --channel=$CHANNEL --device $MODEL --show-image
ubuntu-device-flash $SERVER $REVARGS query --channel=$CHANNEL --device $MODEL --show-image

# Ask the user to confirm, maybe
if [ x"$ASK" = x"yes" ]; then
  echo -n "Press enter to continue, or ^C to exit: "
  read
fi

function rootcmd {
  adb shell "echo $PASSWD | sudo -S $*"
}

# Handle fastboot mode and fastboot recovery mode
if [ x"$FASTBOOT_RECOVERY" != x ]; then
  # FIXME: remove; no longer used?
  echo 'Rebooting to bootloader/fastboot'
  adb reboot bootloader
  sleep 10
  echo "Flashing recovery image $FASTBOOT_RECOVERY"
  fastboot flash recovery $FASTBOOT_RECOVERY
  echo "Rebooting into recovery"
  fastboot boot $FASTBOOT_RECOVERY
  sleep 10
  FASTBOOT=no
elif [ x"$FASTBOOT" = xyes ]; then
  EXTRA="--bootstrap"
  echo 'Rebooting to bootloader/fastboot'
  adb reboot bootloader
  sleep 10
fi

echo 'flashing device'
echo ubuntu-device-flash $SERVER $REVARGS touch --channel=$CHANNEL --device $MODEL $DEVMODE $RECOVERY --wipe $DEVTAR $EXTRA
ubuntu-device-flash $SERVER $REVARGS touch --channel=$CHANNEL --device $MODEL $DEVMODE $RECOVERY --wipe $DEVTAR $EXTRA

# Try to ensure adb is going to work
# (create the access file during the "magic window"
# when recovery.img is still running)
# (add it repeatedly to make sure it happens *after* /userdata gets formatted)
sleep 15
if [ x$ADB = xyes ]; then
  COUNT=10  # try this many times
  ADB_ENABLED=
  while [ "$COUNT" -gt 0 ]; do
    # display progress if we haven't succeeded yet
    if [ -z "$ADB_ENABLED" ]; then
      echo "Trying to add .adb_onlock ($COUNT tries remaining)"
    fi
    # create and verify the file
    if adb shell echo success | grep success > /dev/null ; then
      # no rootcmd is needed since recovery mode is already root
      adb shell touch /data/.adb_onlock
      if adb shell ls /data/.adb_onlock | grep adb_onlock > /dev/null ; then
        ADB_ENABLED=yes
      fi
    fi
    COUNT=$(expr "$COUNT" - 1)
    sleep 6
  done
  # final result
  if [ x"$ADB_ENABLED" = x"yes" ]; then
    echo 'Added .adb_onlock successfully.'
  else
    echo 'Failed to add .adb_onlock.'
  fi
fi

echo 'waiting for device to boot'
adb wait-for-device
echo 'waiting a few more seconds for boot'
sleep 5

# Try again to ensure adb is going to work
# (may require a human to help)
if [ x"$ADB" = x"yes"  -a  x"$ADB_ENABLED" != x"yes" ]; then
  # make sure adb shell works
  FAILED=no
  while ! adb shell echo success | grep success > /dev/null ; do
    echo "Please unlock the screen manually to enable developer mode"
    FAILED=yes
    sleep 3
  done
  # FIXME?: also should copy ~/.android/adb_key.pub to /data/misc/adb/adb_keys
  rootcmd touch /userdata/.adb_onlock
  ADB_ENABLED=yes
  # alternate method:  (if $FAILED)
  # - adb reboot bootloader
  # - fastboot boot device/recovery.img
  # - make sure /data is mounted
  # - touch /data/.adb_onlock
  # - adb reboot
fi
if [ x"$ADB_ENABLED" != x"yes" ]; then
  # No ADB means we can't do anything else, so exit.
  echo 'Adb not enabled or not requested; exiting.'
  if [ x"$BEEP" = x"yes" ]; then beep ; fi
  exit 0
fi

# Override the widget sizes, maybe
if [ x"$GU" != x  -a  x"$GU" != xyes  -a  x"$GU" != xno ]; then
  echo 'Setting better GU value'
  adb shell 'printf "start on starting dbus\nexec initctl set-env --global GRID_UNIT_PX='$GU'\n" > ~/.config/upstart/gu.conf'
fi

# Set up wifi!
# (configure this via phablet-network)
# (or uncomment the other lines here)
if [ x"$WIFI" = xyes ]; then
  echo 'connecting to the network'
  #WIFINAME=$(grep ^name $HOME/.wifi_auth | cut -d '=' -f 2)
  #WIFIPASSWD=$(grep ^passwd $HOME/.wifi_auth | cut -d '=' -f 2)
  #rootcmd nmcli d wifi connect $WIFINAME password $WIFIPASSWD iface wlan0
  phablet-network  # woot, I finally fixed this tool to make it not stupid
fi

# Push some media files to play
# (you'll probably need to configure this manually)
if [ x"$MEDIA" = xyes ]; then
  echo 'pushing music'
  pushd media/music
  for f in * ; do adb push $f /home/phablet/Music ; done
  popd
  adb shell 'cd /home/phablet/Music && chown phablet.phablet *'

  echo 'pushing videos'
  pushd media/video
  for f in * ; do adb push $f /home/phablet/Videos ; done
  popd
  adb shell 'cd /home/phablet/Videos && chown phablet.phablet *'

  echo 'pushing photos'
  pushd media/photos
  for f in * ; do adb push $f /home/phablet/Pictures ; done
  popd
  adb shell 'cd /home/phablet/Pictures && chown phablet.phablet *'
fi

# Maybe push some extra scripts?
#echo 'pushing scripts'
#rootcmd mount -o remount,rw /
#for f in $HOME/touch/scripts/* ; do
#  echo $f
#  adb push $f /home/phablet
#done
#rootcmd 'mount -o remount,ro /'

# Disable the welcome wizard
if [ x"$WIZARD" = x"no" ]; then
  echo 'disabling wizard'
  phablet-config welcome-wizard --disable
  adb wait-for-device
  REBOOT_NEEDED=yes
fi

# Disable the edges intro
if [ x"$EDGEINTRO" = x"no" ]; then
  echo 'disabling edges intro'
  phablet-config edges-intro --disable
  adb wait-for-device
fi

# force an apt-get update
if [ x"$APTUPDATE" = x"yes" ]; then
  echo 'apt-get update'
  rootcmd mount -o remount,rw /
  rootcmd apt-get update
  rootcmd mount -o remount,ro /
  REBOOT_NEEDED=yes
fi

if [ x"$REBOOT_NEEDED" = x"yes" ]; then
  echo 'rebooting to activate previous changes'
  adb reboot
  adb wait-for-device
fi

# All done, beep to notify the user.
if [ x"$BEEP" = x"yes" ]; then beep ; fi
